#include <stdio.h>
#include <hardware/custom.h>
#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <graphics/gfxbase.h>

// 20 instead of 127 because of input.device priority
#define TASK_PRIORITY           (20)
#define COLOR00                 (0x180)
#define BPLCON0                 (0x100)
#define PRA_FIR0_BIT            (1 << 6)
#define BPLCON0_COMPOSITE_COLOR (1 << 9)

extern struct GfxBase *GfxBase;
extern struct Custom custom;

#include "colorUtil.h"
// #include "sineUtil.c"		// ?? decalaration expected ??

void SampleSineWORD(WORD * dst, float offset, float amplitude, UWORD nrSamples)
{
	float deltaAngle = 2.0 * M_PI / ((float) nrSamples), currAngle;
	float currSine;

	UWORD iSample;
	WORD currSample;
 
	for (iSample = 0; iSample < nrSamples; iSample++)
	{
		currAngle = deltaAngle * ((float) iSample);
		currSine = sin(currAngle);
		currSample = (WORD) round(offset + (amplitude * currSine));
		dst[iSample] = currSample;
	}
}

#define BG_COLOR				(0x000)
#define CENTER_LINE				(150)

#define NRCOL01					(3)
#define NRCOL12					(5)
#define NRCOL					((NRCOL01 + NRCOL12 - 1) * 2) - 1
#define COP2LEN					(4 + (4 * NRCOL) + 4 + 2)

static UWORD __chip coplist2[COP2LEN];

void recalcCopList2LinesY(UWORD * dst, UWORD centerY)
{
	UWORD startY = centerY - (NRCOL / 2);
	
	for(UWORD iLine = 0; iLine < NRCOL; iLine++)
	{
		dst[iLine] = startY + iLine;
	}
}

void calcCopList2Colors(UWORD * dst, UWORD col0, UWORD col1, UWORD col2)
{
	SampleGradient(col0, col1, NRCOL01, dst);
	SampleGradient(col1, col2, NRCOL12, dst + NRCOL01 - 1);
	MirrorGradient(dst, dst + (NRCOL01 + NRCOL12 - 2), (NRCOL01 + NRCOL12 - 1));
}

void recalcCopList2(UWORD * linesY, UWORD * colors)
{
	// init
	UWORD iCop = 0;
	coplist2[iCop++] = BPLCON0;
	coplist2[iCop++] = BPLCON0_COMPOSITE_COLOR;
	coplist2[iCop++] = COLOR00;
	coplist2[iCop++] = BG_COLOR;
	
	// actual list
	for(UWORD iLine = 0; iLine < NRCOL; iLine++)
	{
		coplist2[iCop++] = ((linesY[iLine]) << 8) + 7;
		coplist2[iCop++] = 0xfffe;
		coplist2[iCop++] = COLOR00;
		coplist2[iCop++] = colors[iLine];
	}
	
	// lead out
	coplist2[iCop++] = ((linesY[NRCOL - 1] + 1) << 8) + 7;
	coplist2[iCop++] = 0xfffe;
	coplist2[iCop++] = COLOR00;
	coplist2[iCop++] = BG_COLOR;
	
	coplist2[iCop++] = 0xffff;
	coplist2[iCop++] = 0xfffe;
}

BOOL init_display(void)
{
    LoadView(NULL);  // clear display, reset hardware registers
    WaitTOF();       // 2 WaitTOFs to wait for 1. long frame and
    WaitTOF();       // 2. short frame copper lists to finish (if interlaced)
    return (((struct GfxBase *) GfxBase)->DisplayFlags & PAL) == PAL;
}

void reset_display(void)
{
    LoadView(((struct GfxBase *) GfxBase)->ActiView);
    WaitTOF();
    WaitTOF();
    custom.cop1lc = (ULONG) ((struct GfxBase *) GfxBase)->copinit;
    RethinkDisplay();
}

int main(int argc, char **argv)
{
	int nrAnimFrames = 100;		// 2 seconds
	WORD sineTable[nrAnimFrames];
	SampleSineWORD(sineTable, (float) CENTER_LINE, 80.0f, nrAnimFrames);
	int iAnim = 0;
	
	UWORD linesY[NRCOL];
	UWORD centerY = sineTable[iAnim];
	recalcCopList2LinesY(linesY, centerY);
	
	UWORD colors[NRCOL];
	calcCopList2Colors(colors, 0x113, 0x55a, 0x77f);
	
	recalcCopList2(linesY, colors);
	
    SetTaskPri(FindTask(NULL), TASK_PRIORITY);
    BOOL is_pal = init_display();
    custom.cop1lc = (ULONG) coplist2;
	
	BOOL cont = TRUE;
    volatile UBYTE *ciaa_pra = (volatile UBYTE *) 0xbfe001;
	while(cont)		// frame loop
	{
		WaitTOF();
		// ON_VBLANK;
		
		iAnim += 1;
		if(iAnim >= nrAnimFrames) iAnim = 0;
		centerY = sineTable[iAnim];
		recalcCopList2LinesY(linesY, centerY);
		recalcCopList2(linesY, colors);
		
		if((*ciaa_pra & PRA_FIR0_BIT) == 0) cont = FALSE;	// quit when lmb pressed
	}
	
    reset_display();
    return 0;
}

